#!/usr/bin/perl
use strict;
use warnings;
use DBI::DBD;
use Getopt::Long qw(HelpMessage);

our($logtime, $command, $dbfile);
require config;

my($runit, $now);
my $debug = 0;

GetOptions(
	'verbose|v' => \$debug,
) or HelpMessage(-exitval => 2);

# ignore terminated or stopped child processes
$SIG{'CHLD'} = 'IGNORE';

# disable output buffering
$| = 1;

my $pid;

sub msg {
	my $msg = shift;

	if ($logtime == 0) {
		print "$msg\n";
	} else {
		my $stamp = localtime;
		printf "%s %s\n", $stamp, $msg;
	}
}

sub fork_and_run {
	my $pid = fork();
	
	if (not defined $pid) {
		msg "resources not available";
	
	} elsif ($pid == 0) {
		msg "[child] executing: $command";
		exec $command;
	
	} else {
		msg "[parent] child forked: $pid";
		return $pid;
	}
}

sub cleanup {
	msg "[parent] cleaning up";
	if (defined($pid) && kill(0, $pid)) {
		kill(15, $pid);
	}
	exit;
}

msg "[parent] started";

$SIG{'INT'} = \&cleanup;
$SIG{'TERM'} = \&cleanup;

# connect to the database
my $db = DBI->connect("dbi:SQLite2:dbname=$dbfile") or die "db connect failed";

if (!(-w $dbfile)) {
	msg "[parent] error: database is not writable";
	exit;
}
my @args = split(/ /, $command);
if (!(-x $args[0])) {
	msg "[parent] error: $command not executable";
	exit;
}

while(1) {
	$now = time;

	# send the alive signal to db
	$db->prepare("UPDATE general SET backend=$now")->execute;

	# db check: should there be a stream and is it running (running == true?)
	my $sql = $db->prepare("SELECT streamid, start, running, due, finished FROM stream WHERE (start < $now AND due > $now)");
	$sql->execute();
	my($streamid, $start, $running, $due, $finished);
	$sql->bind_columns( undef, \$streamid, \$start, \$running, \$due, \$finished );
	$sql->fetch();
	my $rows = $sql->rows;

	if ($rows == -1) {
		msg "[parent] db: stream sould NOT be running" unless $debug == 0;
		$runit = 'no';
	} elsif ($rows == 1) {
		msg "[parent] db: stream $streamid sould be running" unless $debug == 0;
		$runit = 'yes';
	}

	$sql->finish();

	# live: is the stream running (^^ and should it be running?)
	if (defined($pid) && kill(0, $pid)) {
		msg "[parent] Stream is running (pid $pid)" unless $debug == 0;
		# stop stream
		if ($runit eq 'no') {
			msg "[parent] stopping stream";
			kill(15, $pid);

			$db->prepare("UPDATE stream SET running='false', finished='true' WHERE running='true'")->execute;
		}
	} else {
		# start stream
		if($runit eq 'yes') {
			msg "[parent] starting stream $streamid";
			$pid = fork_and_run();

			$db->prepare("UPDATE stream SET running='true' WHERE streamid=$streamid")->execute;
		}
	}
	
	sleep 20;
}
